# -*- coding: utf8 -*-
from __future__ import unicode_literals

import xml.etree.cElementTree as ET

from driverng import Message


class Driver(Message):
    SPEC = {
        "code": "iq200xml",
        "label": "Iq200 XML",
    }

    def parse(self, data):
        return ET.fromstring(data.encode("utf16"))

    def get_message_type(self, message):
        if message.tag == "SIQ":
            return self.QUERY
        else:
            return self.ORDER

    def to_string(self, message):
        header = '<?xml version="1.0" encoding="UTF-16"?>'
        return header + ET.tostring(message).decode()

    def get_order_message(self, worklist, context):
        root = ET.Element("SI")
        root.set("BF", "URN")
        root.set("WO", "run")
        has_order = False
        for sample in worklist.samples:
            root.text = sample["sample_id"]
            demog = ET.SubElement(root, "DEMOG")
            for order in sample["orders"]:
                # Les suppressions ne sont pas gérées
                if order["action"] == "del":
                    continue
                has_order = True
                demog.set("LName", sample["last_name"])
                demog.set("FName", sample["first_name"])
                demog.set("RecNum", sample["patient_id"])
                demog.set("Gender", sample["sex"])
                demog.set(
                    "DOB",
                    sample["birthdate"].strftime("%Y-%m-%d") if sample["birthdate"] else ""
                )
        if has_order:
            return root

    def get_queries(self, message):
        queries = []
        if message.tag == "SIQ":
            queries.append(message.text)
        return queries

    def get_result_dict(self, message):
        results = {}
        sa = message.tag
        if sa == "SA":
            sample_id = message.get('ID')
            result = results.setdefault(
                sample_id, {"resultats": []})["resultats"]
            cristal = set()
            cylindre = set()

            for ac in message.findall('AC'):
                if ac.get('AT') == 'Sediment':
                    for ar in ac.findall('AR'):
                        test = ar.get('Key')
                        resu = ar.text.split(" ")
                        if test in ("HYAL", "EPIC", "WBCT", "RBCT", "GRAN"):
                            if resu[0] == "[none]":
                                cylindre.add(resu[0])
                            else:
                                cylindre.add("%s%s" % (test, resu[0]))
                        elif test in (
                            "CACB", "CAPH", "CAOX", "TPO4", "URIC", "LEUC",
                            "CYST", "TYRO", "AMOR"
                        ):
                            if resu[0] == "[none]":
                                cristal.add(resu[0])
                            else:
                                cristal.add("%s%s" % (test, resu[0]))
                        result.append({
                            "test": test,
                            "resu": resu[0],
                        })

            for test, resu in (
                ("CYLINDRE", cylindre),
                ("CRISTAL", cristal),
            ):
                if not len(resu):
                    continue
                elif len(resu) > 1 and "[none]" in resu:
                    # Plus d'un resultat != [none]
                    resu.remove("[none]")
                result.append({
                    "test": test,
                    "resu": ";".join(sorted(resu)),
                })
        return results
