# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng.drivers.hprim_spec import (
    HprimSegmentH,
    HprimSegmentP,
    Driver as Hprim
)

from driverng.astm import (
    HprimString,
    HprimMultiple,
    HprimPN,
)


class ITMSegmentH(HprimSegmentH):

    emetteur = HprimMultiple(
        5,
        code=HprimString(1, default="SGL", obligatoire=True),
        nom=HprimString(2, default="MCA", obligatoire=True),
    )
    recepteur = HprimMultiple(
        10,
        code=HprimString(1, default="ITM", obligatoire=True),
        nom=HprimString(2, default="ITM", obligatoire=True),
    )


class ITMSegmentP(HprimSegmentP):
    patient_demandeur = HprimString((3, 1), max_length=40, obligatoire=True)
    patient_executant = HprimString((4, 1), max_length=40, obligatoire=True)
    nom = HprimPN(6, max_length=80, obligatoire=True)
    nomjf = HprimString(7, max_lenth=80, obligatoire=True)


class Driver(Hprim):
    SPEC = {
        "code": "itm",
        "label": "Itm",
    }

    SEGMENT_CLASS_MAP = dict(Hprim.SEGMENT_CLASS_MAP.items() + {
        "H": ITMSegmentH,
        "P": ITMSegmentP,
    }.items())
    DEFAULT_SEPARATORS = "|^~\&"

    def get_order_message(self, worklist, context):
        msg = self.create(
            date=context["now"],
            type_message="ORM",
            mode_traitement="",
        )
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            action={"del": worklist.IGNORE, worklist.ELSE: "N"},
            # FIXME: Ne semble pas supporter plusieurs OBR avec plusieurs
            # urgences.
            priority={"S": "A", "A": "A", worklist.ELSE: "R"},
        ):
            seg_p = msg.patients.add(
                patient_demandeur=patient["patient_id"],
                nomjf=patient["maiden_name"],
                date_nai=patient["birthdate"],
                sexe=patient["sex"],
            )
            seg_p.nom.nom = patient["last_name"]
            seg_p.nom.prenom = patient["first_name"]
            for sample, action, priority in patient["samples"]:
                seg_obr = seg_p.orders.add(
                    tube_demandeur=sample["sample_id"],
                    priorite=priority,
                    no_demandeur=sample["sample_id"],
                    date_prel=sample["collection_date"],
                    action=action,
                    date_recu=sample["request_date"],
                )
                seg_obr.prescripteur.medecin.nom = sample["physician"] or "/"
                seg_obr.prescripteur.service.nom = patient["location"] or "/"
                seg_obr.nature.type_tube.code = sample["specimen"]
                for order in sample["orders"]:
                    test = seg_obr.prescription.add()
                    test.code1 = order["code"]

        if msg.patients:
            return msg
