# -*- coding: utf8 -*-
from __future__ import unicode_literals


from driverng import Message
from driverng.fixed import (
    FixedLengthFormat,
    String,
    Date,
)

SEP = "|"


class ProtisResult(FixedLengthFormat):
    FMT_DESCRIPTION = [
        String("sample_id", 12),
        SEP,
        String("patient_id", 15, right_justified=True),
        SEP,
        String("last_name", 40, right_justified=True),
        SEP,
        String("first_name", 40, right_justified=True),
        SEP,
        Date("birthdate", 10, fmt_date="%d.%m.%Y"),
        SEP,
        String("sex", 1),  # M/F
        SEP,
        Date("date", 13, fmt_date="%d.%m.%Y|%H"),
        SEP,
        String("code", 6, right_justified=True),
        SEP,
        String("flag", 4, right_justified=True, default='    '),
        SEP,
        String("result", 8, right_justified=True),
        SEP,
        String("unit", 7, right_justified=True),
        SEP,
        Date("collection_date", 10, fmt_date="%d.%m.%Y"),
        SEP,
        String("service", 30, right_justified=True),
        SEP,
        String("specimen", 1),
        SEP,
    ]


class Protis(list):

    def add(self, **kwargs):
        obj = ProtisResult(**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        for line in data.splitlines():
            if not line:
                continue
            self.append(ProtisResult().parse(line))
        return self

    def to_string(self):
        return "\r\n".join([e.to_string() for e in self]) + "\r\n"


class Driver(Message):
    SPEC = {
        "code": "protis",
        "label": "Protis",
    }

    def create(self):
        return Protis()

    def parse(self, data):
        return Protis().parse(data)

    def to_string(self, message):
        return message.to_string()

    def get_order_message(self, worklist, context):
        msg = self.create()
        for sample in worklist.samples:
            for result in sample["results"]:
                msg.add(
                    sample_id=sample["sample_id"],
                    patient_id=sample["patient_id"],
                    last_name=sample["last_name"],
                    first_name=sample["first_name"],
                    birthdate=sample["birthdate"],
                    sex=sample["sex"],
                    code=result["code"],
                    date=result["date"],
                    result=result["value"],
                    unit=result["unit"],
                    collection_date=sample["collection_date"],
                    service=sample["service"],
                    specimen=sample["specimen"],
                )
        if len(msg):
            return msg
        else:
            return None
