# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng.drivers.sysmex.base import AstmSysmexBase


class AstmSysmex(AstmSysmexBase):
    SPEC = {
        "code": "sysmex_suit",
        "label": "Sysmex SUIT",
    }
    HANDLE_DELETED = False

    def __init__(self, sysmex_suit_analyseur_format="INST",
                 sysmex_suit_rack_format="%(RACK)s-%(TUBE)s-%(INST)s-%(INID)s",
                 **kwargs):
        self.analyseur_format = sysmex_suit_analyseur_format
        self.rack_format = sysmex_suit_rack_format
        super(AstmSysmex, self).__init__(**kwargs)

    @staticmethod
    def get_lots_reactifs(seg_obx):
        return []

    @staticmethod
    def ignore_segment_obx(seg_obx):
        if seg_obx.test_code in (
            "H_RACK", "U_RACK", "C_RACK",
            "H_TUBE", "U_TUBE", "C_TUBE",
            "H_INID", "U_INID", "C_INID",
            "H_INST", "U_INST", "C_INST",
        ):
            return True
        if seg_obx.status == "F":
            # On ne garde que les résultats finaux
            return False
        else:
            return True

    @staticmethod
    def get_flags(message):
        flags = {}
        for seg_p in message.patients:
            for seg_obr in seg_p.orders:
                for seg_obx in seg_obr.results:
                    if seg_obx.test_code[:5] == "FLAG_":
                        flags.setdefault(
                            seg_obr.tube_executant, {}
                        ).setdefault(
                            seg_obx.test_code[5:],
                            []
                        ).append(seg_obx.valeur)
        return flags

    def get_analyseur(self, analyseur, seg_obx):
        return analyseur

    def get_analyseur_qc(self, message):
        m = ["%s_%s" % (k, self.analyseur_format)
             for k in ("H", "U", "C")]
        for seg_s in message.controls:
            for seg_obx in seg_s.results:
                # Normalement d'après la doc, le H_INST est donné dans un
                # segment S Mais dans les faits il peut être dans un segment
                # OBX
                if seg_obx.test_code in m:
                    return seg_obx.valeur

        for seg_s in message.controls:
            if seg_s.test in m:
                return seg_s.result

        return None

    def get_analyseur_rack(self, seg_obr):
        infos = {
            "RACK": "",  # H_RACK, le rack
            "TUBE": "",  # H_TUBE, position dans le rack
            "INST": "",  # H_INST, nom de l'analyseur
            # H_INID, id de l'analyseur ou zone de tri
            # de l'analyseur de tri (TS-500)
            "INID": "",
        }
        for seg_obx in seg_obr.results:
            # H Pour Hémato
            # U Pour Urine
            # C pour Coagulatioon
            if seg_obx.test_code in ("H_RACK", "U_RACK", "C_RACK"):
                infos["RACK"] = seg_obx.valeur
            if seg_obx.test_code in ("H_TUBE", "U_TUBE", "C_TUBE"):
                infos["TUBE"] = seg_obx.valeur.zfill(3)
            if seg_obx.test_code in ("H_INID", "U_INID", "C_INID"):
                # Dans CAUTO
                # A1983 est remplacé par XE-5000
                # F1944 est remplacé par XE-2100D
                infos["INID"] = seg_obx.valeur
            if seg_obx.test_code in ("H_INST", "U_INST", "C_INST"):
                infos["INST"] = seg_obx.valeur

        rack = self.rack_format % infos
        analyseur = infos[self.analyseur_format] or None
        return analyseur, rack

    def get_alarmes(self, seg_obx):
        alarmes = seg_obx.abnormal_flags[:]
        if (seg_obx.test_code, seg_obx.comment_code) == ("PLT", "W"):
            alarmes.append("O")
        return alarmes
