# -*- coding: utf8 -*-
from __future__ import unicode_literals

import importlib


class LazyImport(object):

    def __init__(self):
        self.modules = {}

    def get_module(self, name):
        if name not in self.modules:
            try:
                mod = importlib.import_module(name)
            except ImportError:
                mod = None
            self.modules[name] = mod
        return self.modules[name]

    @property
    def matplotlib(self):
        if "matplotlib" not in self.modules:
            matplotlib = self.get_module("matplotlib")
            if matplotlib is not None:
                matplotlib.use("agg")
                matplotlib.rc("font", size=6)
            return matplotlib
        else:
            return self.modules["matplotlib"]

    @property
    def pylab(self):
        return self.get_module("matplotlib.pylab")

    @property
    def Image(self):
        return self.get_module("PIL.Image")

    @property
    def ImageDraw(self):
        return self.get_module("PIL.ImageDraw")

    def import_all(self):
        if all([
            self.matplotlib is not None,
            self.pylab is not None,
            self.Image is not None,
            self.ImageDraw is not None,
        ]):
            return True
        else:
            return False


lazy = LazyImport()
