# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng.protocol.astm import AstmProtocol
from driverng.protocol.advia import AdviaProtocol
from driverng.protocol.void import VoidProtocol
from driverng.protocol.mllp import MLLPProtocol
from driverng.protocol.acknack import ACKNACKProtocol
from driverng.protocol.dimension import DimensionProtocol
from driverng.protocol.vscube import VSCubeProtocol
from driverng.protocol.abx import AbxProtocol
from driverng.protocol.tosoh import TosohProtocol
from driverng.protocol.cd3200 import Cd3200Protocol
from driverng.protocol.bci import BCIProtocol
from driverng.protocol.alifax import AlifaxProtocol
from driverng.protocol.phoresys import PhoresysProtocol
from driverng.protocol.rapidlab import RapidlabProtocol
from driverng.protocol.noastm import NoAstmProtocol
from driverng.protocol.kermit import KermitProtocol
from driverng.protocol.lh import LHProtocol
from driverng.protocol.ha81 import Ha81Protocol
from driverng.protocol.au680 import Au680Protocol
from driverng.protocol.pvs import PvsProtocol
from driverng.protocol.bn2 import BN2SerHostProtocol, BN2AutoHostProtocol
from driverng.protocol.sedimax import SedimaxProtocol

PROTOCOLES = {
    "astm": AstmProtocol,
    "advia2120": AdviaProtocol,
    "void": VoidProtocol,
    "mllp": MLLPProtocol,
    "acknack": ACKNACKProtocol,
    "dimension": DimensionProtocol,
    "vscube": VSCubeProtocol,
    "abx": AbxProtocol,
    "tosoh": TosohProtocol,
    "cd3200": Cd3200Protocol,
    "bci": BCIProtocol,
    "alifax": AlifaxProtocol,
    "phoresys": PhoresysProtocol,
    "rapidlab": RapidlabProtocol,
    "noastm": NoAstmProtocol,
    "kermit": KermitProtocol,
    "lh": LHProtocol,
    "ha81": Ha81Protocol,
    "au680": Au680Protocol,
    "pvs": PvsProtocol,
    "bn2_serhost": BN2SerHostProtocol,
    "bn2_autohost": BN2AutoHostProtocol,
    "sedimax": SedimaxProtocol,
}

_LEGACY_PROTOCOLES = {
    "stago": AstmProtocol,
    "mpl": AstmProtocol,
    "sysmex_suit": AstmProtocol,
    "sysmex_suit_plus": AstmProtocol,
    "advia": AdviaProtocol,
    "ctlink": AstmProtocol,
    "isys": AstmProtocol,
    "advia1200": AstmProtocol,
    "advia1800": AstmProtocol,
    "piano": AstmProtocol,
    "hemofax": AstmProtocol,
    "ruby": AstmProtocol,
    "sapphire": AstmProtocol,
    "ca560": ACKNACKProtocol,
    "arch2000": AstmProtocol,
    "versacell": AstmProtocol,
    "centaur": AstmProtocol,
    "eurolab": VoidProtocol,
    "vitros": AstmProtocol,
    "zenit": MLLPProtocol,
    "dimension": DimensionProtocol,
    "vscube": VSCubeProtocol,
    "pentra120": AbxProtocol,
    "tosoh": TosohProtocol,
    "vista": AstmProtocol,
    "cd3200": Cd3200Protocol,
    "immulite": AstmProtocol,
    "alifax": AlifaxProtocol,
    "acltop": AstmProtocol,
    "tango": AstmProtocol,
    "dm": AstmProtocol,
    "cs2x00i": AstmProtocol,
    "vidas": BCIProtocol,
    "scanlab": VoidProtocol,
    "ssa": VoidProtocol,
    "ca660": AstmProtocol,
    "ais": AstmProtocol,
    "liaison": AstmProtocol,
    "phoresys": PhoresysProtocol,
    "m2000": AstmProtocol,
    "immunocap": AstmProtocol,
    "swing": VoidProtocol,
    "rapidlab": RapidlabProtocol,
    "sp1000i": NoAstmProtocol,
    "lxl": AstmProtocol,
    "acl9000": AstmProtocol,
    "hprim": VoidProtocol,
    "dxh": AstmProtocol,
    "variant": AstmProtocol,
    "autovue": VoidProtocol,
    "kone": AstmProtocol,
    "vitros_kermit": KermitProtocol,
    "ha8180": AstmProtocol,
    "biorad": VoidProtocol,
    "streamlab": AstmProtocol,
    "poch": NoAstmProtocol,
    "lh": LHProtocol,
    "kalilab": VoidProtocol,
    "vidasxml": VoidProtocol,
    "bcsxp": AstmProtocol,
    "vitek": BCIProtocol,
    "bcsxp_bctmode": AstmProtocol,
    "ha81": Ha81Protocol,
    "immucor": VoidProtocol,
    "protis": VoidProtocol,
    "gentiane": VoidProtocol,
    "elecsys": AstmProtocol,
    "previisolaxml": VoidProtocol,
    "vt": ACKNACKProtocol,
    "itm": VoidProtocol,
    "vt50": ACKNACKProtocol,
    "kryptor": AstmProtocol,
    "qwalys": AstmProtocol,
    "evolis": AstmProtocol,
    "b221": AstmProtocol,
    "iq200xml": AstmProtocol,
    "cobas": AstmProtocol,
    "au680": Au680Protocol,
    "diana": AstmProtocol,
    "erytra": AstmProtocol,
    "cla1": VoidProtocol,
    "abl": AstmProtocol,
    "helena": AstmProtocol,
    "mylaxml": AstmProtocol,
    "pvs": PvsProtocol,
    "bnpros": AstmProtocol,
    "bn2_serhost": BN2SerHostProtocol,
    "bn2_autohost": BN2AutoHostProtocol,
    "qiagen": MLLPProtocol,
    "remisol": AstmProtocol,
    "meqnet": AstmProtocol,
    "etimax": AstmProtocol,
    "quantalizer": AstmProtocol,
    "genexpert": AstmProtocol,
    "engen": AstmProtocol,
    "sedimax": SedimaxProtocol,
}

_PROTOCOLES_KWARGS = {
    # Le max_car_bloc est géré par le format...
    "advia1200": {"end_payload_with_cr": False, "max_car_bloc": None},
    "advia1800": {"end_payload_with_cr": False, "max_car_bloc": None},
    "versacell": {"max_car_bloc": None},
    # SUIT plus has no limitation in the string length.
    # SUIT plus do not uses "ETB"
    "sysmex_suit_plus": {"max_car_bloc": None},
    "cd3200": {"checksum_type": ACKNACKProtocol.CHECKSUM_ASTM},
    "acltop": {"mode_bloc": True},
    "ha8180": {"mode_etb": True},
    "bcsxp_bctmode": {"end_payload_with_cr": False},
    "vt": {"ack": False},
    "iq200xml": {"mode_iq200": True, "end_payload_with_cr": False},
    "abl": {"mode_etb": True},
    "genexpert": {"mode_bloc": True},
}

_VOID_KWARGS = {
    "qwalys": {"multipart_astm": True},
}


def get_protocole(options, logger):
    type_serv = int(options["type_serv"])
    if type_serv == 5:  # fichier
        klass = VoidProtocol
        kwargs = _VOID_KWARGS.get(options["format_automate"], {})
    else:
        code_automate = options["format_automate"]
        klass = _LEGACY_PROTOCOLES[code_automate]
        kwargs = _PROTOCOLES_KWARGS.get(code_automate, {})

        for b_opt in ("mode_bloc", "mode_query", "is_automate"):
            if b_opt in options:
                kwargs[b_opt] = True

        if "REPERTOIRE_FTP_ADVIA2120" in options and code_automate == "advia":
            kwargs["repertoire_ftp"] = options["REPERTOIRE_FTP_ADVIA2120"]

    kwargs["logger"] = logger
    kwargs["encoding"] = options.get("encoding", "ISO-8859-15")
    return klass(**kwargs)
