# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime
import random

from driverng.protocol.acknack import ACKNACKProtocol


class DimensionProtocol(ACKNACKProtocol):
    POLL_15 = datetime.timedelta(seconds=15)
    POLL_1 = datetime.timedelta(seconds=1)

    def __init__(self, *args, **kwargs):
        self.poll = False
        self.last_poll = None
        self.query = False
        self.poll_timing = self.POLL_15
        kwargs.setdefault("checksum_type", self.CHECKSUM_ASTM)
        super(DimensionProtocol, self).__init__(*args, **kwargs)

    def can_send_trames(self, wqueue):
        if not self.is_automate and (self.poll or self.query):
            if not wqueue.empty():
                return True
            elif not self.query:
                self.send_msg(b"N")  # No request
        if self.is_automate:
            now = datetime.datetime.now()
            if (self.last_poll is None or
                    now - self.last_poll > self.poll_timing):
                if random.randint(0, 10):
                    request = b"1"  # Request
                else:
                    request = b"0"  # Busy
                self.send_msg(b"P\x1cDIM\x1c0\x1c%s\x1c0" % (request,))
                self.last_poll = now
            return True
        return False

    def send_msg(self, data):
        return self.send_trames(self.build_trames(data))

    def build_trames(self, message):
        return super(DimensionProtocol, self).build_trames(b"%s\x1c" % (message,))

    def send_trames(self, trame):
        self.tries = 0
        self.poll = False
        self.query = False
        return super(DimensionProtocol, self).send_trames(trame)

    def recv_trames(self):
        payload = super(DimensionProtocol, self).recv_trames()
        if payload:
            if payload[0] in (b"R", b"D", b"C"):
                # Result/Request/Calibration acceptance
                self.send_msg(b"M\x1cA")
                if self.is_automate and payload[0] == b"D":
                    self.poll_timing = self.POLL_1
            elif payload[0] == b"M":
                # Ignore Result/Request acceptance
                return False
            elif payload[0] == b"I":
                # reponse au query
                self.query = True
            elif payload[0] == b"P":
                data = payload.split(b"\x1c")
                if data[3] == b"0":
                    # Busy, send wait poll
                    self.send_msg(b"W")
                elif data[3] == b"1":
                    self.poll = True
                else:
                    raise NotImplementedError
                return False
            elif self.is_automate and payload[0] in (b"N", b"W"):
                self.poll_timing = self.POLL_15
                return False
        return payload
