# -*- coding: utf8 -*-
from __future__ import unicode_literals

import time
import datetime

from driverng.utils import VT, FS
from driverng.protocol.base import BaseProtocol


class MLLPProtocol(BaseProtocol):
    HL7_ACK = b"""MSH|^~\&|MCA^HOST||S02^ZENIT||%(date)s||%(message_type)s|%(control_id)s|P|2.5\rMSA|AA|%(ack_id)s|\r"""

    def __init__(self, **kwargs):
        super(MLLPProtocol, self).__init__(**kwargs)

    def send_trames(self, trames):
        self.write(trames)
        self.recv_ack()
        return True

    def build_trames(self, message):
        # <vt><payload><fs><cr>
        return b"%c%s\r%c\r" % (
            VT,
            b"\r".join(message.splitlines()),
            FS)

    def mllp_readline(self):
        return self.readline(break_func=lambda d: d and d[-2:] == b"%c\r" % (
            FS,))

    def recv_ack(self):
        # TODO check ack
        self.mllp_readline()

    def recv_trames(self):
        data = self.mllp_readline()
        payload = self.get_payload(data)

        # TODO: Le ack devrait être généré proprement et par MCA
        context = {
            "date": datetime.datetime.now().strftime("%Y%m%d%H%M%S"),
            "control_id": int(time.time() * 1000),
            "ack_id": payload.splitlines()[0].split("|")[9],
        }
        if self.is_automate:
            context["message_type"] = b"ORL^022^ORL_022"
        else:
            context["message_type"] = b"ACK^R01^ACK"
        self.write(self.build_trames(MLLPProtocol.HL7_ACK % context))
        return payload

    @staticmethod
    def get_payload(data):
        return data[1:-2]
