# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng.utils import ACK, NACK, STX, ETX, EOT, byte2log
from driverng.protocol.base import BaseProtocol


class PhoresysProtocol(BaseProtocol):

    def build_trames(self, message):
        # <stx><payload><etx>
        payloads = [l for l in message.splitlines() if l]
        messages = []
        for i, line in enumerate(payloads, 1):
            if i == len(payloads):
                end = EOT
            else:
                end = ETX
            messages.append(b"%c%s%c" % (
                STX, line, end))
        return messages

    def send_trames(self, trames):
        tries = 0
        index = 0
        while True:
            try:
                trame = trames[index]
            except IndexError:
                break
            self.write(trame)
            data = self.read(1, timeout=10)
            data += self.read(1, timeout=10)
            data += self.read(1, timeout=10)
            if (
                len(data) != 3
                or (data[0], data[2]) != (STX, ETX)
                or data[1] not in (ACK, NACK)
            ):
                self.logger.info("Unexpected ack '%s'", byte2log(data))
                return False
            if data[1] == NACK:
                tries += 1
                if tries > 3:
                    return False
                continue
            else:  # ACK
                tries = 0
                index += 1
        return True

    def recv_trames(self):
        data = self.read()
        if not data or data[0] != STX:
            self.logger.warning("<stx> expected in %s", byte2log(data))
            return False
        data += self.readline(break_func=lambda s: s and s[-1] in (ETX, EOT))
        self.write(b"%c%c%c" % (STX, ACK, ETX))
        return data[1:-1]
