# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng.utils import ACK, NACK, STX, ETX, EOT, byte2log
from driverng .protocol.base import BaseProtocol


class TosohProtocol(BaseProtocol):

    def __init__(self, max_tries=4, **kwargs):
        self.max_tries = max_tries
        super(TosohProtocol, self).__init__(**kwargs)

    def send_trames(self, trames):
        for line in trames:
            self.tries = 0
            while True:
                self.tries += 1
                self.write(line)
                data = self.read(timeout=10)
                if not data or data[0] not in (ACK, NACK):
                    self.logger.info(
                        "Unexpected data '%s'", byte2log(data))
                    self.write(EOT)
                    return False
                if data[0] == ACK:
                    break
                if self.tries >= self.max_tries:
                    self.write(EOT)
                    return False
        self.write(EOT)
        return True

    def recv_trames(self):

        lines = []

        while True:
            data = self.read()

            if not data or data[0] not in (STX, EOT):
                self.logger.warning(
                    "<stx> or <eot> expexted in %s", byte2log(data))
                return False

            if data[0] == EOT:
                break

            data += self.readline(
                break_func=lambda s: s and len(s) >= 2 and s[-2] == ETX)
            if self.check_trame(data):
                self.write(ACK)
                lines.append(self.get_payload(data))
            else:
                self.write(NACK)
                return False

        return b"\n".join(lines)

    def checksum(self, data):
        # XOR of <payload><etx>
        ck = 3  # ETX
        for c in data:
            ck ^= ord(c)
        return ck

    def check_trame(self, trame):
        # TODO
        pass
        try:
            checksum = ord(trame[-1])
            checksum_expected = self.checksum(trame[1:-2])
        except ValueError:
            self.logger.warning(
                "Missing checksum for %s",
                byte2log(trame),
            )
            return False

        if checksum != checksum_expected:
            self.logger.warning(
                "Bad checksum for %s: (%02X != %02X)",
                byte2log(trame),
                checksum,
                checksum_expected,
            )
            return False

        return True

    def get_payload(self, data):
        # <stx><payload><etx><checksum>
        return data[1:-2]

    def build_trames(self, message):
        # <stx><payload><etx><checksum>
        data = []
        for line in message.splitlines():
            data.append(b"%c%s%c%c" % (
                STX,
                line,
                ETX,
                self.checksum(line),
            ))
        return data
