# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng.protocol.base import BaseProtocol
from driverng.utils import splitlines


class VoidProtocol(BaseProtocol):

    def __init__(self, multipart_astm=False, **kwargs):
        self.multipart_astm = multipart_astm
        super(VoidProtocol, self).__init__(**kwargs)

    def send_trames(self, trames):
        self.write(trames)
        return True

    def build_trames(self, message):
        if message[-1] not in (b"\r", b"\n"):
            message += b"\r\n"
        return message

    def recv_trames(self):
        message = self.read(size=None)
        if message and message[0] == "H" and self.multipart_astm:
            return self.get_messages(message)
        else:
            return message

    @staticmethod
    def get_messages(message):
        messages = []
        current_message = []

        for line in splitlines(message):
            if line and line[0] == b"H":
                if current_message:
                    messages.append(b"\n".join(current_message))
                    current_message = []

            current_message.append(line)

        if current_message:
            messages.append(b"\n".join(current_message))

        return messages
