# -*- coding: utf8 -*-
from __future__ import unicode_literals

import time

from driverng.utils import byte2log, ACK, NACK
from driverng.protocol.base import BaseProtocol


class VSCubeProtocol(BaseProtocol):

    @staticmethod
    def break_func_ack(buf):
        if len(buf) > 1 and buf[-1] == b"\r":
            return True
        else:
            return False

    @staticmethod
    def break_func(buf):
        if len(buf) > 3 and buf[-3] == b"\r":
            return True
        else:
            return False

    def send_trames(self, trame):

        self.write(trame)

        if self.is_automate:
            # On lis l'ack
            self.readline(break_func=self.break_func_ack)

        return True

    def recv_trames(self):

        data = self.read(1)

        if not data or data[0] != '>':
            return False

        data += self.readline(break_func=self.break_func)

        # Seul le driver envoie des ack...

        if not self.is_automate:
            time.sleep(1)

        if self.check_trame(data) and not self.is_automate:
            self.write(b"%c01\r" % (ACK,))
        elif not self.is_automate:
            # NACK erreur de checksum
            # 00 => Erreur générale
            # 04 => Erreur de checksum
            # 05 => Erreur valeur H-LEN / L-LEN
            # 06 => Erreur longeur du champ de données
            self.write(b"%c0104\r" % (NACK,))

        return self.get_payload(data)

    @staticmethod
    def get_payload(data):
        return data[1:-3]

    def check_trame(self, trame):
        # TODO gestion checksum desactivé
        try:
            checksum = int(trame[-2:], 16)
            checksum_expected = self.checksum(trame[:-2])
        except ValueError:
            self.logger.warning("Missing checksum for %s", byte2log(trame))
            return False

        if checksum != checksum_expected:
            self.logger.warning(
                "Bad checksum for %s: (%02X != %02X)",
                byte2log(trame),
                checksum,
                checksum_expected,
            )
            return False

        return True

    def build_trames(self, message):
        # <'>'><payload><cr><checksum>
        message = message.splitlines()[0]
        data = b">%s\r" % (message)
        return b"%s%02X" % (
            data,
            self.checksum(data),
        )

    @staticmethod
    def checksum(data):
        chk = 0
        for c in data:
            chk ^= ord(c)
        return chk
