# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng.transport.rs232 import SerialTransport
from driverng.transport.tcp import TCPTransport
from driverng.transport.fichier import FileTransport

# Type serv
# 0 -> serial
# 5 -> file
# 6 -> tcp/ip
_TRANSPORT_KWARGS = {
    # (format_automate, type_serv) -> kwargs
    ("cla1", 5): {"append": True},
}


def get_transport(options, logger):
    type_serv = int(options["type_serv"])
    kwargs = _TRANSPORT_KWARGS.get((options["format_automate"], type_serv), {})
    kwargs["logger"] = logger
    if type_serv == 0:
        if options["nom_port"][0] != "/":
            kwargs["port"] = "/dev/%s" % options["nom_port"]
        else:
            kwargs["port"] = options["nom_port"]

        kwargs["baudrate"] = int(options["baud"])
        if options["parite"].lower() == "true":
            kwargs["parity"] = "O"

        kwargs["bytesize"] = int(options["n_bit"])
        kwargs["stopbits"] = int(options["stop_bit"])
        if options["xon_in"].lower() == "true":
            kwargs["xonxoff"] = True
        else:
            kwargs["xonxoff"] = False
        return SerialTransport(**kwargs)
    elif type_serv == 6:
        if options["parite"].lower() == "true":
            server = True
        else:
            server = False
        host, port = options["nom_port"].split(":")
        port = int(port)
        return TCPTransport(host, port, server, **kwargs)
    elif type_serv == 5:  # Fichier
        input_dir = options["nom_port"]
        if "fic_in" in options:
            output_dir, fmt = options["fic_in"].rsplit("/", 1)
        elif "output_dir" in options:
            output_dir, fmt = options["output_dir"], None
        glob = options["fic_out"]
        return FileTransport(input_dir, output_dir, fmt, glob, **kwargs)
    else:
        raise NotImplementedError("transport type %s not implemented" % (
            type_serv,
        ))
