# -*- coding: utf8 -*-
from __future__ import unicode_literals

import select

from driverng.utils import byte2log


class TransportException(Exception):
    pass


class BaseTransport(object):

    def __init__(self, logger=None):
        super(BaseTransport, self).__init__()
        self.logger = logger
        self.meta = {}

    def write(self, data):
        self.logger.debug("send %s", byte2log(data))
        return self._write(data)

    def _write(self, data):
        raise NotImplementedError

    def read(self, size):
        data = self._read(size)
        self.logger.debug("recv %s", byte2log(data))
        return data

    def _read(self, size):
        raise NotImplementedError

    def readline(self, break_func=lambda buf: buf[-2:] == b"\r\n"):
        """Lecture d'un bloc de données

        @param break_func: Fonction appelée à chaque bloc lu pour savoir
        s'il faut continuer de lire ou non. Par défaut c'est les caractères
        <cr><lf> en fin de buffer (protocole ASTM)
        """

        data = b""
        while not break_func(data):
            data += self._read(4096)

        self.logger.debug("recv %s", byte2log(data))
        return data

    def poll(self, timeout, wqueue):
        r, _, _ = select.select([self.fileno(), ], [], [], timeout)
        if r:
            return True
        else:
            return False

    def flush(self):
        while True:
            if self.poll(2, None):
                self.read(1024)
            else:
                break

    def fileno(self):
        raise NotImplementedError

    def set_timeout(self, timeout):
        pass

    def close(self):
        pass

    def post_send(self):
        self.meta = {}

    def post_recv(self):
        pass

    def pre_send(self, meta):
        self.meta = meta or {}

    def pre_recv(self):
        pass
